package com.czsj.wechat.service.impl;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.czsj.common.utils.DateUtils;
import com.czsj.wechat.entity.WxAccount;
import com.czsj.wechat.utils.PageUtils;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.redis.WxRedisOps;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import me.chanjar.weixin.mp.config.impl.WxMpRedisConfigImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.czsj.wechat.mapper.CzsjWxAccountMapper;
import com.czsj.wechat.domain.CzsjWxAccount;
import com.czsj.wechat.service.ICzsjWxAccountService;

/**
 * 微信公众号Service业务层处理
 * 
 * @author czsj
 * @date 2024-12-07
 */
@Slf4j
@Service
public class CzsjWxAccountServiceImpl implements ICzsjWxAccountService 
{

    @Autowired
    private WxMpService wxMpService;

    @Autowired
    private WxRedisOps wxRedisOps;

    @Autowired
    private CzsjWxAccountMapper czsjWxAccountMapper;



    @Override
    public void loadWxMpConfigStorages() {
        List<CzsjWxAccount> accountList = czsjWxAccountMapper.selectCzsjWxAccountList(null);
        if (accountList == null || accountList.isEmpty()) {
            log.info("未读取到公众号配置，请在管理后台添加");
            return;
        }
        log.info("加载到{}条公众号配置",accountList.size());
        accountList.forEach(this::addAccountToRuntime);
    }

    @Override
    public boolean removeByIds(Collection<?> idList) {
        return false;
    }

    /**
     * 查询微信公众号
     * 
     * @param uid 微信公众号主键
     * @return 微信公众号
     */
    @Override
    public CzsjWxAccount selectCzsjWxAccountByUid(Long uid)
    {
        return czsjWxAccountMapper.selectCzsjWxAccountByUid(uid);
    }

    /**
     * 查询微信公众号列表
     * 
     * @param czsjWxAccount 微信公众号
     * @return 微信公众号
     */
    @Override
    public List<CzsjWxAccount> selectCzsjWxAccountList(CzsjWxAccount czsjWxAccount)
    {
        return czsjWxAccountMapper.selectCzsjWxAccountList(czsjWxAccount);
    }

    /**
     * 新增微信公众号
     * 
     * @param czsjWxAccount 微信公众号
     * @return 结果
     */
    @Override
    public int insertCzsjWxAccount(CzsjWxAccount czsjWxAccount)
    {
        czsjWxAccount.setCreateTime(DateUtils.getNowDate());
        addAccountToRuntime(czsjWxAccount);
        return czsjWxAccountMapper.insertCzsjWxAccount(czsjWxAccount);
    }

    /**
     * 修改微信公众号
     * 
     * @param czsjWxAccount 微信公众号
     * @return 结果
     */
    @Override
    public int updateCzsjWxAccount(CzsjWxAccount czsjWxAccount)
    {
        czsjWxAccount.setUpdateTime(DateUtils.getNowDate());
        addAccountToRuntime(czsjWxAccountMapper.selectCzsjWxAccountByUid(czsjWxAccount.getUid()));
        return czsjWxAccountMapper.updateCzsjWxAccount(czsjWxAccount);
    }

    /**
     * 批量删除微信公众号
     * 
     * @param uids 需要删除的微信公众号主键
     * @return 结果
     */
    @Override
    public int deleteCzsjWxAccountByUids(Long[] uids)
    {
        for (Long uid : uids) {
            removeAccountToRuntime(czsjWxAccountMapper.selectCzsjWxAccountByUid(uid));
        }
        return czsjWxAccountMapper.deleteCzsjWxAccountByUids(uids);
    }

    /**
     * 删除微信公众号信息
     * 
     * @param uid 微信公众号主键
     * @return 结果
     */
    @Override
    public int deleteCzsjWxAccountByUid(Long uid)
    {
        return czsjWxAccountMapper.deleteCzsjWxAccountByUid(uid);
    }

    /**
     * 添加账号到当前程序，如首次添加需初始化configStorageMap
     * @param entity
     */
    private synchronized void addAccountToRuntime(CzsjWxAccount entity) {
        String appid = entity.getAppId();
        WxMpDefaultConfigImpl config = buildWxMpConfigImpl(entity);
        wxMpService.addConfigStorage(appid, config);
    }

    /**
     * 移除当前账号到当前程序
     * @param entity
     */
    private synchronized void removeAccountToRuntime(CzsjWxAccount entity) {
        String appid = entity.getAppId();
        wxMpService.removeConfigStorage(appid);
    }


    private WxMpDefaultConfigImpl buildWxMpConfigImpl(CzsjWxAccount entity) {
        log.info("添加公众号配置到程序！公众号appID:" + entity.getAppId());
        WxMpRedisConfigImpl configStorage = new WxMpRedisConfigImpl(wxRedisOps, "v1:@ss.hasPermi(mp");
        configStorage.setAppId(entity.getAppId());
        configStorage.setSecret(entity.getSecret());
        configStorage.setToken(entity.getToken());
        configStorage.setAesKey(entity.getAesKey());
        return configStorage;
    }
}
